# User.pm - The Object Class that provides a User Object
# Created by James A. Pattie, 2004-06-25.

# Copyright (c) 2000-2004, Xperience, Inc. http://www.pcxperience.com/
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.

package Portal::CompanyAdmin::User;
use strict;
use Portal::AppState;
use Portal::Forms::User;
use Portal::Forms::Preferences;
use Portal::XML::ConfigParser;
use Portal::XML::ConfigObject;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::AppState Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.1';

# global variables.
# modify to represent the commands this state provides.
# the name => value have the following meaning:
# name is the command this state recognizes.
# value is the langObj entry to use for the title when processing that command.
my %commands = ( display => "selectUser", viewUser => "userInfo", editUser => "EditUserInfo", changePassword => "changePassword", editPreferences => "UserPreferences", deleteUser => "DeleteUser", createUser => "CreateUser", editAppPreferences => "EditAppUserPreferences" );

=head1 NAME

User - Object used to build a User Object Class.

=head1 SYNOPSIS

  use Portal::CompanyAdmin::User;
  my $obj = Portal::CompanyAdmin::User->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

User is a User class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new()

 Creates a new instance of the Portal::CompanyAdmin::User module.
 See Portal::AppState(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{commands} = \%commands;
  $self->{state} = "User";

  my $arguments = $self->arguments();
  eval "\$self->{userForms} = Portal::Forms::User->new($arguments);";
  if ($@)
  {
    $self->error($@);
    return $self;
  }
  if ($self->{userForms}->error)
  {
    $self->error($self->{userForms}->errorMessage);
    return $self;
  }

  eval "\$self->{preferencesForms} = Portal::Forms::Preferences->new($arguments);";
  if ($@)
  {
    $self->error($@);
    return $self;
  }
  if ($self->{preferencesForms}->error)
  {
    $self->error($self->{preferencesForms}->errorMessage);
    return $self;
  }


  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->buildUrls;

  return $self;
}

=item % getCommands(void)

  This method returns the commands hash and is only to be
  used by the registerApp method for creating the permissions,
  if using the AppStateCommandSecurity implementation.

=cut
sub getCommands
{
  return %commands;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item HTMLObject c_display()

=cut
sub c_display
{
  my $self = shift;
  my $doc = $self->prepDoc();
  if ($self->error)
  {
    $self->prefixError();
    return undef;
  }

  my $url = $self->url;

  # call the Portal::Forms::User module to do the form display work.
  my %output = $self->{userForms}->selectUserAdminForm(companyObj => $self->{sessionObj}->{store}->{companyObj},
      url => $url, app => $self->{app}, state => $self->{state} );
  if ($self->{userForms}->error)
  {
    $self->error($self->{userForms}->errorMessage);
    return undef;
  }

  # now display the output to the user.
  $doc->printTag(tag => "<maincontent>", value => $output{body});
  delete $output{body};
  # output the rest of the form info.
  $doc->print(%output);

  return $doc;
}

=item HTMLObject c_viewUser()

=cut
sub c_viewUser
{
  my $self = shift;
  my $doc = $self->prepDoc();
  if ($self->error)
  {
    $self->prefixError();
    return undef;
  }

  # make sure we have a uname specified.
  if (!exists $self->{input}->{uname})
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("missingArgument"), "uname");
    $doc = $self->c_display();
    return $doc;
  }
  elsif (length $self->{input}->{uname} == 0)
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("invalidArgument"), "uname", "");
    $doc = $self->c_display();
    return $doc;
  }
  my $uname = $self->{input}->{uname};

  # lookup the specified user.
  my $tmpUser = $self->{authObj}->getUserInfo(uname => $uname);
  if ($self->{authObj}->error)
  {
    $self->error($self->{authObj}->errorMessage);
    return undef;
  }
  if (!defined $tmpUser)
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("userNotFound"), $uname);
    $doc = $self->c_display();
    return $doc;
  }

  # call the Portal::Forms::User module to do the core display, validation and update work
  my %output = $self->{userForms}->viewUser(user => $tmpUser, companyObj => $self->{sessionObj}->{store}->{companyObj});
  if ($self->{userForms}->error)
  {
    $self->error($self->{userForms}->errorMessage);
    return undef;
  }

  # now display the output to the user.
  $doc->printTag(tag => "<maincontent>", value => $output{body});
  delete $output{body};
  # output the rest of the form info.
  $doc->print(%output);

  return $doc;
}

=item HTMLObject c_editUser

=cut
sub c_editUser
{
  my $self = shift;
  my $doc = $self->prepDoc();
  if ($self->error)
  {
    $self->prefixError();
    return undef;
  }
  my $successEditPhrase = $self->langObj->map("successUserEdit");

  my $url = $self->url;

  # make sure we have a uname specified.
  if (!exists $self->{input}->{uname})
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("missingArgument"), "uname");
    $doc = $self->c_display();
    return $doc;
  }
  elsif (length $self->{input}->{uname} == 0)
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("invalidArgument"), "uname", "");
    $doc = $self->c_display();
    return $doc;
  }
  my $uname = $self->{input}->{uname};

  # lookup the specified user.
  my $tmpUser = $self->{authObj}->getUserInfo(uname => $uname);
  if ($self->{authObj}->error)
  {
    $self->error($self->{authObj}->errorMessage);
    return undef;
  }
  if (!defined $tmpUser)
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("userNotFound"), $uname);
    $doc = $self->c_display();
    return $doc;
  }

  # call the Portal::Forms::User module to do the core display, validation and update work
  my @result = $self->{userForms}->editUser(user => $tmpUser,
      url => $url, app => $self->{app}, state => $self->{state});

  if ($self->{userForms}->error)
  {
    $self->{input}->{errorMessage} = $self->{userForms}->errorMessage;
    $self->{userForms}->clearErrors("all");
    $self->{userForms}->resetError;
    $doc = $self->c_display();
    return $doc;
  }

  # now display the output to the user.
  if ($result[0] == 2)
  {
    $doc->printTag(tag => "<maincontent>", value => $result[1]->{body});
    delete $result[1]->{body};
    # output the rest of the form info.
    $doc->print(%{$result[1]});
    $doc->setOnload(code => $result[3]);
  }
  elsif ($result[0] == 1)
  {
    # update the session for our userObj.
    if ($uname eq $self->{userObj}->{uname})
    {
      $self->{portalSession}->setMode("write");
      $self->{portalSession}->{store}->{userObj} = $result[2];
      $self->{portalSession}->setMode("read");
    }

    # let the user know the edit suceeded.
    $self->{input}->{statusMessage} = $successEditPhrase;
    $doc = $self->c_display();
    return $doc;
  }

  return $doc;
}

=item HTMLObject c_changePassword

=cut
sub c_changePassword
{
  my $self = shift;
  my $doc = $self->prepDoc();
  if ($self->error)
  {
    $self->prefixError();
    return undef;
  }
  my $successPasswordPhrase = $self->langObj->map("successPassword");

  my $url = $self->url;

  # make sure we have a uname specified.
  if (!exists $self->{input}->{uname})
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("missingArgument"), "uname");
    $doc = $self->c_display();
    return $doc;
  }
  elsif (length $self->{input}->{uname} == 0)
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("invalidArgument"), "uname", "");
    $doc = $self->c_display();
    return $doc;
  }
  my $uname = $self->{input}->{uname};

  # lookup the specified user.
  my $tmpUser = $self->{authObj}->getUserInfo(uname => $uname);
  if ($self->{authObj}->error)
  {
    $self->error($self->{authObj}->errorMessage);
    return undef;
  }
  if (!defined $tmpUser)
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("userNotFound"), $uname);
    $doc = $self->c_display();
    return $doc;
  }

  # call the Portal::Forms::User module to do the core display, validation and update work
  my @result = $self->{userForms}->changePassword(user => $tmpUser,
      url => $url, app => $self->{app}, state => $self->{state});

  if ($self->{userForms}->error)
  {
    $self->{input}->{errorMessage} = $self->{userForms}->errorMessage;
    $self->{userForms}->clearErrors("all");
    $self->{userForms}->resetError;
    $doc = $self->c_display();
    return $doc;
  }

  # now display the output to the user.
  if ($result[0] == 2)
  {
    $doc->printTag(tag => "<maincontent>", value => $result[1]->{body});
    delete $result[1]->{body};
    # output the rest of the form info.
    $doc->print(%{$result[1]});
    $doc->setOnload(code => $result[2]);
  }
  elsif ($result[0] == 1)
  {
    $self->{input}->{statusMessage} = $successPasswordPhrase;
    $doc = $self->c_display();
    return $doc;
  }

  return $doc;
}

=item HTMLObject c_deleteUser()

=cut
sub c_deleteUser
{
  my $self = shift;
  my $doc = $self->setupCSS();
  if ($self->error)
  {
    $self->prefixError();
    return undef;
  }
  my $successDeletePhrase = sprintf($self->langObj->map("successUserDelete"), $self->{input}->{uname});

  my $url = $self->url;

  # make sure we have a uname specified.
  if (!exists $self->{input}->{uname})
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("missingArgument"), "uname");
    $doc = $self->c_display();
    return $doc;
  }
  elsif (length $self->{input}->{uname} == 0)
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("invalidArgument"), "uname", "");
    $doc = $self->c_display();
    return $doc;
  }
  my $uname = $self->{input}->{uname};

  # lookup the specified user.
  my $tmpUser = $self->{authObj}->getUserInfo(uname => $uname);
  if ($self->{authObj}->error)
  {
    $self->error($self->{authObj}->errorMessage);
    return undef;
  }
  if (!defined $tmpUser)
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("userNotFound"), $uname);
    $doc = $self->c_display();
    return $doc;
  }

  if ($tmpUser->{uname} eq $self->{userObj}->{uname})
  {
    $self->{input}->{errorMessage} = $self->{langObj}->map("YouCanNotDeleteYourself");
    $doc = $self->c_display();
    return $doc;
  }

  # call the Portal::Forms::User module to do the update work
  my $result = $self->{userForms}->deleteUser(userObj => $tmpUser, companyObj => $self->{sessionObj}->{store}->{companyObj});

  if ($self->{userForms}->error)
  {
    $self->{input}->{errorMessage} = $self->{userForms}->errorMessage;
    $self->{userForms}->clearErrors("all");
    $self->{userForms}->resetError;
    $doc = $self->c_display();
    return $doc;
  }

  # now display the output to the user.
  if ($result->{result} == 1)
  {
    $self->{input}->{statusMessage} = $successDeletePhrase . $result->{message};
    $doc = $self->c_display();
    return $doc;
  }

  return $doc;
}

=item HTMLObject c_createUser

=cut
sub c_createUser
{
  my $self = shift;
  my $doc = $self->prepDoc();
  if ($self->error)
  {
    $self->prefixError();
    return undef;
  }
  my $successCreatePhrase = sprintf($self->langObj->map("successUserCreate"), $self->{input}->{uname});
  my $companyObj = $self->{sessionObj}->{store}->{companyObj};

  my $url = $self->url;

  # call the Portal::Forms::User module to do the core display, validation and create work
  my @result = $self->{userForms}->createUser(companyObj => $companyObj, url => $url, app => $self->{app}, state => $self->{state});

  if ($self->{userForms}->error)
  {
    $self->{input}->{errorMessage} = $self->{userForms}->errorMessage;
    $self->{userForms}->clearErrors("all");
    $self->{userForms}->resetError;
    $doc = $self->c_display();
    return $doc;
  }

  # now display the output to the user.
  if ($result[0] == 2)
  {
    $doc->printTag(tag => "<maincontent>", value => $result[1]->{body});
    delete $result[1]->{body};
    # output the rest of the form info.
    $doc->print(%{$result[1]});
    $doc->setOnload(code => $result[3]);
  }
  elsif ($result[0] == 1)
  {
    # let the user know the create suceeded.
    $self->{input}->{statusMessage} = $successCreatePhrase . "<br />" . $result[4];
    $doc = $self->c_display();
    return $doc;
  }

  return $doc;
}

=item HTMLObject c_editPreferences()

=cut
sub c_editPreferences
{
  my $self = shift;
  my $doc = $self->prepDoc();
  if ($self->error)
  {
    $self->prefixError();
    return undef;
  }

  my $url = $self->url;

  # make sure we have a uname specified.
  if (!exists $self->{input}->{uname})
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("missingArgument"), "uname");
    $doc = $self->c_display();
    return $doc;
  }
  elsif (length $self->{input}->{uname} == 0)
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("invalidArgument"), "uname", "");
    $doc = $self->c_display();
    return $doc;
  }
  my $uname = $self->{input}->{uname};

  # lookup the specified user.
  my $tmpUser = $self->{authObj}->getUserInfo(uname => $uname);
  if ($self->{authObj}->error)
  {
    $self->error($self->{authObj}->errorMessage);
    return undef;
  }
  if (!defined $tmpUser)
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("userNotFound"), $uname);
    $doc = $self->c_display();
    return $doc;
  }

  # call the Portal::Forms::Preferences module to do the core display work.
  my %form = $self->preferencesForms->displayApps(user => $tmpUser, url => $url,
    app => $self->app, state => $self->state);
  if ($self->preferencesForms->error)
  {
    $self->{input}->{errorMessage} = $self->{preferencesForms}->errorMessage;
    $doc = $self->c_display();
    return $doc;
  }

  $doc->printTag(tag => "<maincontent>", value => $form{body});
  delete $form{body};
  # output the rest of the form info.
  $doc->print(%form);

  return $doc;
}

=item HTMLObject c_editAppPreferences()

=cut
sub c_editAppPreferences
{
  my $self = shift;
  my $doc = $self->prepDoc();
  if ($self->error)
  {
    $self->prefixError();
    return undef;
  }
  my $successEditPhrase = $self->langObj->map("successUserPreferenceEdit");

  my $url = $self->url;

  # make sure we have a uname specified.
  if (!exists $self->{input}->{uname})
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("missingArgument"), "uname");
    $doc = $self->c_display();
    return $doc;
  }
  elsif (length $self->{input}->{uname} == 0)
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("invalidArgument"), "uname", "");
    $doc = $self->c_display();
    return $doc;
  }
  my $uname = $self->{input}->{uname};

  # lookup the specified user.
  my $tmpUser = $self->{authObj}->getUserInfo(uname => $uname);
  if ($self->{authObj}->error)
  {
    $self->error($self->{authObj}->errorMessage);
    return undef;
  }
  if (!defined $tmpUser)
  {
    $self->{input}->{errorMessage} = sprintf($self->{langObj}->map("userNotFound"), $uname);
    $doc = $self->c_display();
    return $doc;
  }

  my $appName = $self->input->{appName};

  # call the Portal::Forms::Preferences->editApp() method to do the work of display, validate, update.
  my @result = $self->preferencesForms->editApp(user => $tmpUser, url => $url,
    app => $self->get("app"), state => $self->get("state"), appName => $appName);
  if ($self->preferencesForms->error)
  {
    $self->{input}->{errorMessage} = $self->{preferencesForms}->errorMessage;
    $doc = $self->c_display();
    return $doc;
  }

  if ($result[0] == 2)
  {
    $doc->printTag(tag => "<maincontent>", value => $result[1]->{body});
    delete $result[1]->{body};
    # output the rest of the form info.
    $doc->print(%{$result[1]});
  }
  elsif ($result[0] == 1)
  {
    # let the user know the edit suceeded.
    $self->{input}->{statusMessage} = $successEditPhrase;
    $doc = $self->c_editPreferences();
    return $doc;
  }

  return $doc;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::CompanyAdmin(3), Portal::AppState(3)

=cut
